/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 2.9 (Ratio-of-uniforms), p. 35 
 *
 * this file: algo02_09.c
 *
 * For a demo how to use this algorithm see the file "demo_algo02_09.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

double generator02_09(double (*pdf)(double), double mu, double um, double up, double vp)
/*
 * Algorithm 2.9 (Ratio-of-uniforms), p. 35 
 *
 * pdf... function pointer to the density
 * mu ... constant (should be close to the mode of the distribution)
 * um ... u- left border of (minimal) bounding rectangle 
 * up ... u+ right border of (minimal) bounding rectangle 
 * vp ... upper border of (minimal) bounding rectangle 
 */
{ double u,v,x;

/*1*/  while(1){ 
/*2*/    u = um+(up-um)*uniform();
         v = uniform()*vp;    
/*3*/    x = u/v+mu;
/*4*/    if(v*v <= (*pdf)(x)) return x;
       }
}




