/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 9.6 (Increasing Hazard Rate dynamic composition) p.200 
 *
 * this file: algo09_06.c
 *
 * For a demo how to use this algorithm see the file "demo_algo09_06.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"

double generator09_06(double (*h)(double), double p0)
/*
 * Algorithm 9.6 (Increasing Hazard Rate dynamic composition) p.200 
 *
 * h   ... function pointer to the bounded, decreasing hazard rate h(x)
 * p0  ... design parameter of the algorithm, can be chosen close to the expectation
 *         of the respective distribution
 */
{      double x, u, v, e, lamda0, lamda1, p1;
       int i;  

/*1*/  x = 0.;
       i = 1;
/*2*/  lamda0 = (*h)(p0);
/*3*/  do{
	 i++;
/*4*/	 u = uniform();
/*5*/	 e = -log(1.-u)/lamda0;
/*6*/	 x = x + e;
/*7*/	 v = lamda0*uniform();
/*8*/  }while(v > (*h)(x));
/*9*/  if(x <= p0) 
/*10*/   return x;
/*11*/ p1 = x;
/*12*/ x = p0;
       i = 0;
/*13*/ lamda1 = (*h)(p1)-(*h)(p0);
/*14*/ do{
	 i++;
/*15*/	 u = uniform();
/*16*/	 e = -log(1.-u)/lamda1;
/*17*/	 x = x + e;
/*18*/	 v = lamda0+lamda1*uniform();
/*19*/ }while(v > (*h)(x));
/*20*/ if(x <= p1) 
/*21*/   return x;
/*22*/ else 
/*23*/   return p1;
}




