/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag
 * 
 * Code for Algorithm 10.9 (Discrete-Hazard-Rate-Inversion), p. 235
 *
 * this file: algo10_09.c
 *
 * For a demo how to use this algorithm see the file "demo_algo10_09.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"


int generator10_09(double (*h)(int))
/*
 * Algorithm 10.9 (Discrete-Hazard-Rate-Inversion), p. 235
 * generates discrete variates
 *
 * h... function pointer to the hazard rate of a discrete distribution
 *
 */
{ 
       double p, P, u;
       int ix;

/*1*/  u = uniform();
/*2*/  ix = 0;
       p = (*h)(0);
       P = p;
/*3*/  while(u > P){
/*4*/	 p = p*(*h)(ix+1)*(1-(*h)(ix))/(*h)(ix);
/*5*/	 P+=p;
	 ix++;
       }
/*6*/  return ix;

}
