/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Code for Algorithm 12.9 (Chessboard-Bivariate), p. 335 
 *
 * this file: algo12_09.c
 *
 * For a demo how to use this algorithm see the file "demo_algo12_09.c".
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "uniform.h"
#include "algo.h"

 static int nt;
 static double p, rhoh; 


int setup12_09(double rho)
/*
 * Algorithm 12.9 (Chessboard-Bivariate), p. 335 
 *
 * rho .... Correlation 
 *
 */
{
      double rho1, rho2;

      rhoh = rho;
/*1*/ nt=(int)(sqrt(1./(1.-fabs(rho))));
/*2*/ rho1 = 1.-1./(nt*nt);
      rho2 = 1.-1./((nt+1)*(nt+1));
/*3*/ p = (fabs(rho)-rho2)/(rho1-rho2);
      return 1;
}

void generator12_09(double res[2])
#define U res[0]
#define V res[1]
{     
      double W;
/*4*/ U = uniform();
      V = uniform();
      W = uniform();
/*5*/ if( W < p ) 
/*6*/   V=((int)(nt*U)+V)/nt;
/*7*/ else 
/*8*/   V=((int)((nt+1)*U)+V)/(nt+1.);
/*9*/ if( rhoh < 0. ) 
/*10*/  V = 1.-V;
}
