/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 6.4 (Simple Ratio-of-Uniforms method, p. 138)
 *
 * this file: demo_algo06_04.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo06_04 demo_algo06_04.c algo06_04.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"


/* PDF of the distribution and its derivative (dpdf)
   you can insert your own distribution here
*/

double my_pdf(double x)
{ if(0<=x && x<=1.) return 2.-x;
  else return 0.;}
/* triangular on (0,1) */


#define CLAS 500 /*sets number of classes for the chi-square test*/

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and conducts a chi-square test */
{ int i;
 double x;

 setseed(13324821); /*set seed for uniform generator*/ 

 printf("Demo for Algorithm 6.4\nfirst we display ten numbers generated from my_pdf()\n");
 setup06_04(my_pdf,1.,0.,0.);/* calls set-up*/
 for(i=0;i<10;i++){
   x = generator06_04();
   printf("x %f\n",x);
 }

 printf("\nChi-square test with sample size %d for the normal distribution\n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;
 setup06_04(pdf_normal,1.,0., 0.5);/* calls set-up*/
 for(i=0;i<CLAS*CLAS;i++){
   x = generator06_04();
   obs[(int)(cdf_normal(x)*CLAS)]++;
 }

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}

