/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag/monograph
 * 
 * Demo how to use Algorithm 12.3 (Resampling-ELK), p. 311 
 *
 * this file: demo_algo12_03.c
 *
 * to compile this file you can use the command:
 *
 * cc -o algo12_03 demo_algo12_03.c algo12_03.c uniform.c statutil.c -lm
 *
 */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "algo.h"
#include "statutil.h"
#include "uniform.h"






#define CLAS 500 /*sets number of classes for the chi-square test*/

static long int obs[CLAS];

int main()
/* demonstrates the use of the algorithm
   and conducts a chi-square test */
{ int i;
 double y,x[7]={0.,0.,1.,2.,3.,4.,5.};

 setseed(133248); /*set seed for uniform generator*/

 

 printf("Demo for Algorithm 12.3\n");
 printf("first we display ten numbers generated from the empirical distribution\n");
 for(i=0;i<10;i++){
   y = generator12_03(x,6);
   printf("y %f\n",y);
 }
 
 printf("\nNow we make a chi-square test with sample size %d to test uniformity\n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;

 for(i=0;i<CLAS*CLAS;i++){
   y = generator12_03(x,6);
   obs[(int)(0.2*y*CLAS)]++;
 }

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}

