/*
 * Hormann, Leydold, Derflinger
 * Automatic Nonuniform Random Variate Generation
 * Springer, Heidelberg, New York, 2003
 * http://statistik.wu-wien.ac.at/arvag
 * 
 * Code for Example 2.15, p. 29, 
 * "1/x density" on (1,3) by composition-rejection with constant hat and recycling 
 *
 * this file: example02_15.c
 *
 * to compile this file you can use the command:
 *
 * cc -o example02_15 example02_15.c uniform.c statutil.c -lm
 *
 */


#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "statutil.h"
#include "uniform.h"

/**********************************************************************/

double example02_15(void)
/* 
 * generates random variates from the "1/x density" on (1,3) by 
 * composition-rejection with constant hat and squeeze and recycling 
 */
{ double b[3],w1,w2,u,x,v,y,h[3],s[3];
  int j;

/*1*/  b[0]=1.; 
       b[1]=sqrt(3.); 
       b[2]=3.; 
       w1=sqrt(3.)-1.; 
       w2=(3.-sqrt(3.))/sqrt(3.);
/*2*/  h[1]=1.; 
       h[2]=1./sqrt(3.); 
       s[1]=1./sqrt(3.); 
       s[2]=1./3.;
/*3*/  while(1){ 
/*4*/    u = (w1+w2)*uniform();
/*5*/    if(u<w1){
/*6*/      j=1;
/*7*/      u=u/w1;}
/*8*/    else{ 
/*9*/      j=2;
/*10*/     u=(u-w1)/w2;}
/*11*/   x = b[j-1]+u*(b[j]-b[j-1]);
/*12*/   v = uniform();
/*13*/   y = v*h[j];
/*14*/   if(y<=s[j]) 
/*15*/     return x;
/*16*/   if(y<=1/x) 
/*17*/     return x;
       }
}

/*****************************************************************/


#define CLAS 500

static long int obs[CLAS*CLAS];

int main()
{ int i;
 double x;

 setseed(13324885); /*set seed for uniform generator*/

 printf("Example 2.15:\n\nfirst we display ten generated numbers\n");

 for(i=0;i<10;i++){
   x = example02_15();
   printf("x %f\n",x);
 }
 
 printf("\nNow we make a chi-square test with sample size %d\n\n", CLAS*CLAS);
 for(i=0;i<CLAS;i++) obs[i]=0;

 for(i=0;i<CLAS*CLAS;i++){
   x = example02_15();
   obs[(int)(cdf_oox1_3(x)*CLAS)]++;
 }

 chi2test(obs,CLAS,CLAS*CLAS,1); 
 printf("\n");
 return 0;
}






